@@ -1,3 +1,4 @@
+/* -*- Mode: C; tab-width: 4; indent-tabs-mode: t; c-basic-offset: 4 -*- */
 /* NetworkManager Wireless Applet -- Display wireless access points and allow user control
  *
  * Dan Williams <dcbw@redhat.com>
@@ -24,6 +25,9 @@
 #include <dbus/dbus-glib.h>
 #include <dbus/dbus-glib-lowlevel.h>
 #include <string.h>
+
+#include <glib/gi18n.h>
+
 #include "nm-utils.h"
 #include "applet.h"
 #include "nma-marshal.h"
@@ -157,9 +161,13 @@ request_name (DBusGProxy *proxy, int flags, const char *detail)
 	}
 
 	if (request_name_result != DBUS_REQUEST_NAME_REPLY_PRIMARY_OWNER) {
-		nm_warning ("Could not acquire the %s service as it is already "
-		            "taken.  Return: %d",
-		            detail, request_name_result);
+		if (request_name_result == DBUS_REQUEST_NAME_REPLY_EXISTS)
+			fprintf (stdout, _("An instance of nm-applet is already running.\n"));
+		else {
+			fprintf (stdout, _("Could not acquire the %s service. (%d)\n"),
+			         detail,
+			         request_name_result);
+		}
 		return FALSE;
 	}
 
