@@ -75,22 +75,6 @@ populate_ui (CEPageDsl *self, NMConnection *connection)
 
 	/* Grab password from keyring if possible */
 	str = nm_setting_pppoe_get_password (setting);
-	if (!str) {
-		GError *error = NULL;
-		GValue *value;
-
-		secrets = nm_gconf_get_keyring_items (connection,
-		                                      nm_setting_get_name (NM_SETTING (setting)),
-		                                      FALSE,
-		                                      &error);
-		if (secrets) {
-			value = g_hash_table_lookup (secrets, NM_SETTING_PPPOE_PASSWORD);
-			if (value)
-				str = g_value_get_string (value);
-		} else if (error)
-			g_error_free (error);
-	}
-
 	if (str)
 		gtk_entry_set_text (priv->password, str);
 
@@ -136,7 +120,10 @@ finish_setup (CEPageDsl *self, gpointer unused, GError *error, gpointer user_dat
 }
 
 CEPage *
-ce_page_dsl_new (NMConnection *connection, GtkWindow *parent_window, GError **error)
+ce_page_dsl_new (NMConnection *connection,
+                 GtkWindow *parent_window,
+                 const char **out_secrets_setting_name,
+                 GError **error)
 {
 	CEPageDsl *self;
 	CEPageDslPrivate *priv;
@@ -175,10 +162,8 @@ ce_page_dsl_new (NMConnection *connection, GtkWindow *parent_window, GError **er
 	}
 
 	g_signal_connect (self, "initialized", G_CALLBACK (finish_setup), NULL);
-	if (!ce_page_initialize (parent, NM_SETTING_PPPOE_SETTING_NAME, error)) {
-		g_object_unref (self);
-		return NULL;
-	}
+
+	*out_secrets_setting_name = NM_SETTING_PPPOE_SETTING_NAME;
 
 	return CE_PAGE (self);
 }
