@@ -88,6 +88,7 @@ static char *ignored_phrases[] = {
 	"Business Mobile Networks BV",
 	"Mobile Broadband Minicard Composite Device",
 	"Mobile Communications AB",
+	"(PC-Suite Mode)",
 	NULL
 };
 
@@ -196,152 +197,6 @@ utils_get_device_description (NMDevice *device)
 	return description;
 }
 
-struct cf_pair {
-	guint32 chan;
-	guint32 freq;
-};
-
-static struct cf_pair a_table[] = {
-	/* A band */
-	{  7, 5035 },
-	{  8, 5040 },
-	{  9, 5045 },
-	{ 11, 5055 },
-	{ 12, 5060 },
-	{ 16, 5080 },
-	{ 34, 5170 },
-	{ 36, 5180 },
-	{ 38, 5190 },
-	{ 40, 5200 },
-	{ 42, 5210 },
-	{ 44, 5220 },
-	{ 46, 5230 },
-	{ 48, 5240 },
-	{ 50, 5250 },
-	{ 52, 5260 },
-	{ 56, 5280 },
-	{ 58, 5290 },
-	{ 60, 5300 },
-	{ 64, 5320 },
-	{ 100, 5500 },
-	{ 104, 5520 },
-	{ 108, 5540 },
-	{ 112, 5560 },
-	{ 116, 5580 },
-	{ 120, 5600 },
-	{ 124, 5620 },
-	{ 128, 5640 },
-	{ 132, 5660 },
-	{ 136, 5680 },
-	{ 140, 5700 },
-	{ 149, 5745 },
-	{ 152, 5760 },
-	{ 153, 5765 },
-	{ 157, 5785 },
-	{ 160, 5800 },
-	{ 161, 5805 },
-	{ 165, 5825 },
-	{ 183, 4915 },
-	{ 184, 4920 },
-	{ 185, 4925 },
-	{ 187, 4935 },
-	{ 188, 4945 },
-	{ 192, 4960 },
-	{ 196, 4980 },
-	{ 0, -1 }
-};
-
-static struct cf_pair bg_table[] = {
-	/* B/G band */
-	{ 1, 2412 },
-	{ 2, 2417 },
-	{ 3, 2422 },
-	{ 4, 2427 },
-	{ 5, 2432 },
-	{ 6, 2437 },
-	{ 7, 2442 },
-	{ 8, 2447 },
-	{ 9, 2452 },
-	{ 10, 2457 },
-	{ 11, 2462 },
-	{ 12, 2467 },
-	{ 13, 2472 },
-	{ 14, 2484 },
-	{ 0, -1 }
-};
-
-guint32
-utils_freq_to_channel (guint32 freq)
-{
-	int i = 0;
-
-	while (a_table[i].chan && (a_table[i].freq != freq))
-		i++;
-	if (a_table[i].chan)
-		return a_table[i].chan;
-
-	i = 0;
-	while (bg_table[i].chan && (bg_table[i].freq != freq))
-		i++;
-	return bg_table[i].chan;
-}
-
-guint32
-utils_channel_to_freq (guint32 channel, char *band)
-{
-	int i = 0;
-
-	if (!strcmp (band, "a")) {
-		while (a_table[i].chan && (a_table[i].chan != channel))
-			i++;
-		return a_table[i].freq;
-	} else if (!strcmp (band, "bg")) {
-		while (bg_table[i].chan && (bg_table[i].chan != channel))
-			i++;
-		return bg_table[i].freq;
-	}
-
-	return 0;
-}
-
-guint32
-utils_find_next_channel (guint32 channel, int direction, char *band)
-{
-	size_t a_size = sizeof (a_table) / sizeof (struct cf_pair);
-	size_t bg_size = sizeof (bg_table) / sizeof (struct cf_pair);
-	struct cf_pair *pair = NULL;
-
-	if (!strcmp (band, "a")) {
-		if (channel < a_table[0].chan)
-			return a_table[0].chan;
-		if (channel > a_table[a_size - 2].chan)
-			return a_table[a_size - 2].chan;
-		pair = &a_table[0];
-	} else if (!strcmp (band, "bg")) {
-		if (channel < bg_table[0].chan)
-			return bg_table[0].chan;
-		if (channel > bg_table[bg_size - 2].chan)
-			return bg_table[bg_size - 2].chan;
-		pair = &bg_table[0];
-	} else {
-		g_assert_not_reached ();
-		return 0;
-	}
-
-	while (pair->chan) {
-		if (channel == pair->chan)
-			return channel;
-		if ((channel < (pair+1)->chan) && (channel > pair->chan)) {
-			if (direction > 0)	
-				return (pair+1)->chan;
-			else
-				return pair->chan;
-		}
-		pair++;
-	}
-	return 0;
-}
-
 /*
  * utils_ether_addr_valid
  *
@@ -387,7 +242,7 @@ utils_check_ap_compatible (NMAccessPoint *ap,
 	const char *setting_mode;
 	const char *setting_band;
 	NM80211Mode ap_mode;
-	guint32 freq;
+	guint32 freq, channel;
 
 	g_return_val_if_fail (NM_IS_ACCESS_POINT (ap), FALSE);
 	g_return_val_if_fail (NM_IS_CONNECTION (connection), FALSE);
@@ -426,25 +281,31 @@ utils_check_ap_compatible (NMAccessPoint *ap,
 	setting_band = nm_setting_wireless_get_band (s_wireless);
 	if (setting_band) {
 		if (!strcmp (setting_band, "a")) {
-			if (freq < 5170 || freq > 5825)
+			if (freq < 4915 || freq > 5825)
 				return FALSE;
 		} else if (!strcmp (setting_band, "bg")) {
-			if (freq < 2412 || freq > 2472)
+			if (freq < 2412 || freq > 2484)
 				return FALSE;
 		}
 	}
 
-	// FIXME: channel check
+	channel = nm_setting_wireless_get_channel (s_wireless);
+	if (channel) {
+		guint32 ap_chan = nm_utils_wifi_freq_to_channel (freq);
+
+		if (channel != ap_chan)
+			return FALSE;
+	}
 
 	s_wireless_sec = (NMSettingWirelessSecurity *) nm_connection_get_setting (connection,
-															    NM_TYPE_SETTING_WIRELESS_SECURITY);
+	                                                                          NM_TYPE_SETTING_WIRELESS_SECURITY);
 
 	return nm_setting_wireless_ap_security_compatible (s_wireless,
-											 s_wireless_sec,
-											 nm_access_point_get_flags (ap),
-											 nm_access_point_get_wpa_flags (ap),
-											 nm_access_point_get_rsn_flags (ap),
-											 nm_access_point_get_mode (ap));
+	                                                   s_wireless_sec,
+	                                                   nm_access_point_get_flags (ap),
+	                                                   nm_access_point_get_wpa_flags (ap),
+	                                                   nm_access_point_get_rsn_flags (ap),
+	                                                   nm_access_point_get_mode (ap));
 }
 
 static gboolean
@@ -478,7 +339,7 @@ connection_valid_for_wired (NMConnection *connection,
 		if (!setting_mac)
 			return TRUE;
 
-		str_mac = nm_device_ethernet_get_hw_address (ethdev);
+		str_mac = nm_device_ethernet_get_permanent_hw_address (ethdev);
 		g_return_val_if_fail (str_mac != NULL, FALSE);
 
 		bin_mac = ether_aton (str_mac);
@@ -517,7 +378,7 @@ connection_valid_for_wireless (NMConnection *connection,
 		const char *str_mac;
 		struct ether_addr *bin_mac;
 
-		str_mac = nm_device_wifi_get_hw_address (wdev);
+		str_mac = nm_device_wifi_get_permanent_hw_address (wdev);
 		g_return_val_if_fail (str_mac != NULL, FALSE);
 
 		bin_mac = ether_aton (str_mac);
